import logging

from cocos.sprite import Sprite

from bitflags import cfg
from bitflags.data import img


bit_images = {
    cfg.BIT_NOT_READY: img('bit_none.png'),
    cfg.BIT_TRUE: img('bit_true.png'),
    cfg.BIT_FALSE: img('bit_false.png')
}


class Bit(Sprite):
    def __init__(self, number, status=cfg.BIT_NOT_READY):
        self.number = number
        self.status = status
        super(Bit, self).__init__(self.get_bit_image())

    def get_bit_image(self):
        return bit_images[self.status]

    def activate(self):
        self.status = cfg.BIT_TRUE
        self.update_image()

    def is_on(self):
        return self.status == cfg.BIT_TRUE

    def toggle(self):
        if self.status == cfg.BIT_NOT_READY:
            logging.debug('not ready')
            return
        self.update_image()

    def update_image(self):
        if self.status == cfg.BIT_TRUE:
            self.status = cfg.BIT_FALSE
        else:
            self.status = cfg.BIT_TRUE

        self.image = self.get_bit_image()


class SensorBit(Bit):
    def __init__(self, sensor, number, status=cfg.BIT_NOT_READY):
        super(SensorBit, self).__init__(number, status)
        self.sensor = sensor
        self.sensor.visible = False

    def toggle(self):
        super(SensorBit, self).toggle()
        if self.status == cfg.BIT_TRUE:
            self.sensor.turn_on()
        else:
            self.sensor.turn_off()


class ColorBit(Bit):
    def __init__(self, sensor, number, status=cfg.BIT_NOT_READY):
        super(ColorBit, self).__init__(number, status)
        self.sensor = sensor

    def toggle(self):
        super(ColorBit, self).toggle()
        if self.status == cfg.BIT_TRUE:
            setattr(self.sensor, self.get_color_attr(), 1)
        else:
            setattr(self.sensor, self.get_color_attr(), 0)
        self.sensor.update_image()


class RBit(ColorBit):
    def get_color_attr(self):
        return 'r'


class GBit(ColorBit):
    def get_color_attr(self):
        return 'g'


class BBit(ColorBit):
    def get_color_attr(self):
        return 'b'


class ShootBit(Bit):
    def __init__(self, begin, end, number, status=cfg.BIT_NOT_READY):
        super(ShootBit, self).__init__(number, status)
        self.on_shooting_begin = begin
        self.on_shooting_end = end

    def toggle(self):
        super(ShootBit, self).toggle()
        if self.status == cfg.BIT_TRUE:
            self.on_shooting_begin()
        else:
            self.on_shooting_end()
