import random

from cocos.sprite import Sprite

from bitflags.data import anim


def dummy_callback():
    pass


class Explosion(Sprite):
    def __init__(self, position, on_end_callback=dummy_callback):
        self.on_end_callback = on_end_callback

        super(Explosion, self).__init__(self.get_animation())
        self.position = position
        if self.is_random_rotation():
            self.rotation = random.random() * 360.0

        self.schedule_interval(
            self.on_animation_end, self.get_frames() * self.get_step()
        )

    def get_animation(self):
        return anim(
            self.get_filename(), 1, self.get_frames(), self.get_step(), False
        )

    def on_animation_end(self):
        self.unschedule(self.on_animation_end)
        self.kill()
        self.on_end_callback()

    def get_frames(self):
        return 6

    def get_step(self):
        return 0.1

    def is_random_rotation(self):
        return False


class PointExplosion(Explosion):
    def get_filename(self):
        return 'point-explode.png'

    def is_random_rotation(self):
        return True


class GateExplosion(Explosion):
    def get_filename(self):
        return 'gate-explode.png'
