import pymunk

from cocos.sprite import Sprite

from bitflags.rgb import RGB
from bitflags import cfg
from bitflags.data import img
from bitflags.fx import PointExplosion, GateExplosion


class Gate(Sprite):
    def __init__(self, r, g, b, position, rotation, on_cshape_removed):
        super(Gate, self).__init__(img('gates-limits.png'))
        self.on_cshape_removed = on_cshape_removed

        self.position = position
        self.rotation = rotation

        self.p1 = Point(r, g, b)
        self.p1.position = (-cfg.GATE_HLEN, 0.0)
        self.add(self.p1)

        self.p2 = Point(r, g, b)
        self.p2.position = (cfg.GATE_HLEN, 0.0)
        self.add(self.p2)

        self.init_physics()

        self.health = 2

    def init_physics(self):
        a = self.point_to_world((-cfg.GATE_HLEN, 0.0)).xy
        b = self.point_to_world((cfg.GATE_HLEN, 0.0)).xy

        gate_shape = pymunk.Segment(pymunk.Body(), a, b, 4)
        gate_shape.collision_type = cfg.COLL_RM
        self.cshape = gate_shape

        p1_shape = pymunk.Circle(pymunk.Body(), cfg.GATE_POINT_R)
        p1_shape.collision_type = cfg.COLL_TA
        p1_shape.on_hit = self.on_p1_hit
        p1_shape.rgb = self.p1.rgb()
        p1_shape.body.position = a
        self.p1.cshape = p1_shape

        p2_shape = pymunk.Circle(pymunk.Body(), cfg.GATE_POINT_R)
        p2_shape.collision_type = cfg.COLL_TA
        p2_shape.rgb = self.p2.rgb()
        p2_shape.on_hit = self.on_p2_hit
        p2_shape.body.position = b
        self.p2.cshape = p2_shape

        self.cshapes = [gate_shape, p1_shape, p2_shape]

    def on_p1_hit(self, space):
        self.on_point_hit(space, self.p1)
        self.add(PointExplosion(self.p1.position, self.kill_p1))
        self.try_to_destroy(space)

    def on_p2_hit(self, space):
        self.on_point_hit(space, self.p2)
        self.add(PointExplosion(self.p2.position, self.kill_p2))
        self.try_to_destroy(space)

    def on_point_hit(self, space, point):
        self.health -= 1
        point.visible = False

        space.remove(point.cshape)
        self.cshapes.remove(point.cshape)
        self.on_cshape_removed(point.cshape)

    def kill_p1(self):
        self.p1.kill()
        self.p1 = None
        self.try_to_anim_explode()

    def kill_p2(self):
        self.p2.kill()
        self.p2 = None
        self.try_to_anim_explode()

    def try_to_destroy(self, space):
        if self.health <= 0:
            space.remove(self.cshape)
            self.on_cshape_removed(self.cshape)

    def try_to_anim_explode(self):
        if self.health <= 0:
            self.opacity = 0.0
            self.add(GateExplosion((0, 0), self.kill))


point_images = {
    #'000': img('gates-000.png'),
    '100': img('gates-100.png'),
    '010': img('gates-010.png'),
    '011': img('gates-011.png'),
    # TODO: finish
}


class Point(RGB):
    def get_image_map(self):
        return point_images
