import pyglet

from cocos.director import director
from cocos.layer import Layer, ColorLayer
from cocos.text import Label

from bitflags import cfg


class StartLayer(ColorLayer):
    is_event_handler = True

    def __init__(self, next_scene):
        super(StartLayer, self).__init__(*cfg.BACK_COLOR)
        self.next_scene = next_scene
        self.add(Label(
            cfg.START_TEXT, cfg.CENTER,
            color=cfg.START_LABEL_COLOR,
            font_name='Munro',
            font_size=cfg.START_LABEL_SIZE,
            anchor_x='center', anchor_y='center',
            width=cfg.W * 0.9,
            multiline=True
        ))

    def on_key_press(self, key, modifiers):
        director.replace(self.next_scene())


class EndLayer(ColorLayer):
    def __init__(self):
        super(EndLayer, self).__init__(*cfg.BACK_COLOR)
        self.add(Label(
            cfg.END_TEXT, cfg.CENTER,
            color=cfg.END_LABEL_COLOR,
            font_name='Munro',
            font_size=cfg.END_LABEL_SIZE,
            anchor_x='center', anchor_y='center',
            width=cfg.W * 0.9,
            multiline=True
        ))


class BackgroundLayer(ColorLayer):
    def __init__(self, room):
        super(BackgroundLayer, self).__init__(*cfg.BACK_COLOR)
        self.add(room)

    def replace_room(self, next_room):
        for i, c in self.children[:]:
            self.remove(c)
        self.add(next_room)


class GameObjectsLayer(Layer):
    def __init__(self, player):
        super(GameObjectsLayer, self).__init__()
        self.add(player)


class UILayer(Layer):
    is_event_handler = True

    def __init__(self, bits):
        super(UILayer, self).__init__()

        self.bits = bits

        self.key_to_bit = {}

        self.init_key_to_bit()
        self.add_bits()

    def init_key_to_bit(self):
        for bit in self.bits:
            key = '_{0}'.format(bit.number)
            num_key = 'NUM_{0}'.format(bit.number)
            self.key_to_bit[getattr(pyglet.window.key, key)] = bit
            self.key_to_bit[getattr(pyglet.window.key, num_key)] = bit

    def add_bits(self):
        for i, bit in enumerate(self.bits):
            x_delta = i * cfg.UI_SPACING
            if i > 3:
                x_delta += cfg.UI_MIDDLE_SPACCING
            bit.position = (cfg.UI_PADDING_LEFT + x_delta,
                            cfg.H - cfg.UI_PADDING_TOP)
            self.add(bit)

    def on_key_press(self, key, modifiers):
        bit = self.key_to_bit.get(key)
        if bit:
            bit.toggle()
