import random
import logging

import pymunk
from pymunk.vec2d import Vec2d

from bitflags import cfg
from bitflags.rgb import RGB
from bitflags.data import img


sensor_images = {
    '000': img('sensor-000.png'),

    '001': img('sensor-001.png'),
    '010': img('sensor-010.png'),
    '100': img('sensor-100.png'),

    '011': img('sensor-011.png'),
    '110': img('sensor-110.png'),
    '101': img('sensor-101.png'),

    '111': img('sensor-111.png'),
}


class Sensor(RGB):
    def __init__(self, r, g, b):
        super(Sensor, self).__init__(r, g, b)
        self.image_anchor_y = 0
        self.position = Vec2d(0.0, 12.0)

        self.init_physics()

        self.targets = []

    def init_physics(self):
        points = [
            (-8, 12),
            (+8, 12),

            (+84, 256),
            (-84, 256),
        ]
        self.cbody = pymunk.Body(1, 1)
        self.cbody.position = cfg.PL_INIT_POS
        self.cshape = pymunk.Poly(self.cbody, points)
        self.cshape.sensor = True
        self.cshape.collision_type = cfg.COLL_SE

    def get_target(self):
        rgb = self.rgb()
        for t in self.targets:
            if t.rgb == rgb:
                return t
        return None

    def fallow_player(self, player):
        self.cbody.position = player.cbody.position
        self.cbody.angle = player.cbody.angle

    def get_image_map(self):
        return sensor_images

    def turn_on(self):
        self.schedule_interval(self.flicker, 0.1)
        self.visible = True
        self.scale_x = 1.0

    def turn_off(self):
        self.unschedule(self.flicker)
        self.visible = False

    def flicker(self, dt):
        if any(self.rgb()):
            self.scale_x = 1.0
            self.visible = True
            return

        if random.random() < 0.8:
            self.scale_x *= -1.0
        if random.random() < 0.3:
            self.visible = not self.visible

    def on_sees_target(self, space, arbiter):
        shape = arbiter.shapes[1]
        if not shape in self.targets:
            self.targets.append(shape)
        logging.debug("t {0}".format(len(self.targets)))
        return 0

    def on_looses_target(self, space, arbiter):
        shape = arbiter.shapes[1]
        p = shape.body.position
        if not self.cshape.point_query(p):
            if shape in self.targets:
                self.targets.remove(shape)
        logging.debug("t {0}".format(len(self.targets)))
        return 0
